﻿using Kivii;
using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Lims.Extensions;
using Kivii.Linq;
using Kivii.Text;
using Njhg.Third.Semir.Extensions;
using Njhg.Third.Semir.Transforms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Njhg.Third.Semir.Entities
{
    public class SemirOrderBase : IEntityIsThirdReportOrder
    {
        private JsonServiceClient _client = null;

        public Report OnInstanceThirdReport(Guid ThirdReportOrderKvid)
        {
            //return null;
            if (ThirdReportOrderKvid == Guid.Empty) return null;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return null;
            if (third.ThirdType != typeof(SemirOrder).FullName) return null;
            if (third.Metadata == null) return null;
            var semir = third.Metadata.DicToObject();

            var report = new Report();
            report.BizId = third.SerialNumber;
            report.BizKvid = third.Kvid;
            report.BizType = typeof(ThirdReportOrder).FullName;
            report.ReportId = $"{semir.BRAND_DISPLAY}{DateTime.Now:yyyyMMddHHmmss}";
            //contract.OrderID = semir.DOC_NO;
            report.Type = "B";
            report.TypeEx = "W";
            report.TypeName = "委托送样";
            report.Category = "SE";
            report.DemanderName = semir.COMPANY_NAME;
            report.DemanderContactName = semir.CONTACT_PERSON;
            report.DemanderContactNumber = semir.CONTACT_PHONE;
            report.DemanderContactAddress = semir.COMPANY_ADDRESS;

            report.SampleManufacturerName = semir.APPLY_ORG_NAME;
            report.SampleManufacturerContactName = semir.PAYER_CONTACT_PERSON;
            report.SampleManufacturerContactNumber = semir.PAYER_CONTACT_PHONE;
            report.SampleManufacturerAddress = semir.PAYER_ORG_ADDRESS;

            report.SampleName = semir.MAT_NAME;
            report.SampleBrand = semir.BRAND_DISPLAY;
            report.SampleGradeSafety = semir.CHECK_LEVEL;
            report.SampleGrade = semir.SAMPLE_GRADE;

            report.SampleBatchNumber = semir.FG_MAT_CODE;

            report.Metadata['ProductColor'] = semir.FG_COLOR;
            report.Metadata['MaterialColor'] = semir.COLOR;
            report.Metadata['MaterialNumber'] = semir.MAT_CODE;
            report.Metadata['SampleType'] = semir.THIRDPARTY_TESTING_TESTPART;
            report.Metadata['VersionNumber'] = semir.MAT_DEV_CODE;
            report.Metadata['YearQuarter'] = semir.YEAR+"/"+semir.SEASONS;

            var details = new List<string>();
            foreach (var detail in semir.ITEMS)
            {
                details.Add($"{detail.ITEM_NAME}({detail.ITEM_STANDARD})");
            }
            var tasks = string.Join("、", details.ToArray());
            report.Metadata = new Dictionary<string, string>();
            report.Metadata.Add("ERemark", tasks);
            return report;
        }

        public bool OnRejectThirdReport(Guid ThirdReportOrderKvid)
        {
            return false;
        }
        public bool OnUploadThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(SemirOrder).FullName) return false;
            //订单还未受理的 则自动去找相关报告进行关联
            if (!third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return true;
            var thirdDetails = conn.Select<ThirdReportOrderDetail>(o => o.ThirdOrderKvid == third.Kvid);
            if (thirdDetails.IsNullOrEmpty()) return false;
            var reports = conn.SelectByIds<Report>(thirdDetails.ConvertAll(o => o.ReportKvid));
            if (reports.IsNullOrEmpty()) return false;
            if (reports.Exists(o => o.Status < (int)ReportStatusType.ReportIssue)) return false;

            var result_info = new ReportResultInfo();
            result_info.DOC_NO = third.SerialNumber;
            result_info.URL_PDF = new List<UrlPdf>();

            var rpt_detail = new List<ReportDetail>();

            //针对多个报告存在不合格的项目 则回传订单整体判定为不合格
            var infoJudement = true;
            foreach (var report in reports)
            {
                //if (thirdDetail.ReportKvid == Guid.Empty) continue;
                //var report = reports.FirstOrDefault(o => o.Kvid == thirdDetail.ReportKvid);
                //if (report == null) continue;
                if (report.Status < (int)ReportStatusType.ReportIssue) continue;
                var queryReportItems = conn.From<ReportItem>();
                queryReportItems.Where(o => o.ReportKvid == report.Kvid);
                var allReportItems = conn.Select(queryReportItems);
                var primaryAllReportItems = allReportItems.filterReportItem();
                primaryAllReportItems.ForEach(o => o.PackageUnit = string.Empty);
                var selfItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();
                var partItems = allReportItems.Where(o => o.Type == ReportItemType.SamplePart).ToList();
                //合并色号和部位
                if (!selfItems.IsNullOrEmpty())
                {
                    foreach (var item in selfItems)
                    {
                        var primarys = primaryAllReportItems.Where(o => o.RootKvid == item.RootKvid).ToList();
                        foreach (var primary in primarys)
                        {
                            primary.PackageUnit = string.Empty;
                            primary.PackageUnit = item.Title;
                        }
                    }
                }
                if (!partItems.IsNullOrEmpty())
                {
                    foreach (var item in partItems)
                    {
                        var primarys = primaryAllReportItems.Where(o => o.ParentKvid == item.Kvid).ToList();
                        foreach (var primary in primarys)
                        {
                            primary.TitlePart = string.Empty;
                            primary.TitlePart = item.Title;
                        }
                    }
                }

                #region result_info
                result_info.EXT_DOC_NO = report.ReportId;
                result_info.CHECK_FEE += report.Amount + report.AmountTest;
                result_info.RESULT = 4;
                if (report.TestJudgement == "合格") result_info.RESULT = 1;
                if (report.TestJudgement == "不合格")
                {
                    infoJudement = false;
                    result_info.RESULT = 2;
                }
                result_info.RESULT_TEXT = report.TestJudgement;
                if (!report.TestNeedJudge) result_info.RESULT_TEXT = "此报告无需判定";

                var code = report.Kvid.ToString().Substring(0, 4);
                var internalCode = $"{code}/{report.ReportId}";

                var urlPdf = new UrlPdf();
                urlPdf.RPT_NO = report.ReportId;
                urlPdf.URL = $"{Configs._pdfUrl}/Lims/Report/{internalCode}";
                urlPdf.RESULT = 4;
                if (report.TestJudgement == "合格") urlPdf.RESULT = 1;
                if (report.TestJudgement == "不合格") urlPdf.RESULT = 2;
                urlPdf.RPT_DESC = report.TestJudgement;
                if (!report.TestNeedJudge) urlPdf.RPT_DESC = "此报告无需判定";
                result_info.URL_PDF.Add(urlPdf);
                #endregion

                #region rpt_detail
                var reportDetail = new ReportDetail();
                reportDetail.RPT_NO = report.ReportId;
                reportDetail.URL = $"{Configs._pdfUrl}/Lims/Report/{internalCode}";
                reportDetail.RESULT = 4;
                if (report.TestJudgement == "合格") reportDetail.RESULT = 1;
                if (report.TestJudgement == "不合格") reportDetail.RESULT = 2;
                reportDetail.RPT_DESC = report.TestJudgement;
                if (!report.TestNeedJudge) reportDetail.RPT_DESC = "此报告无需判定";

                reportDetail.ITEMS = new List<Detail>();
                foreach (var item in primaryAllReportItems)
                {
                    var nodeItems = allReportItems.Where(o => o.ParentKvid == item.Kvid).ToList();

                    #region 拼接检测要求及检测值
                    var spec2List = new List<string>();
                    var resultList = new List<string>();
                    if (item.Type == ReportItemType.Test)
                    {
                        spec2List.Add($"{item.Title}({item.DetectionValueQualified})");
                        resultList.Add($"{item.Title}({item.TestValue})");
                    }
                    else if (!nodeItems.IsNullOrEmpty())
                    {
                        foreach (var node in nodeItems)
                        {
                            spec2List.Add($"{node.Title}({node.DetectionValueQualified})");
                            resultList.Add($"{node.Title}({node.TestValue})");
                        }
                    }
                    var spec2 = string.Join(", ", spec2List.ToArray());
                    var result = string.Join(", ", resultList.ToArray());
                    #endregion

                    var detail = new Detail();
                    detail.ITEM_NAME = item.Title;
                    detail.ITEM_DESC = item.Title;
                    detail.TEMP_ITEM_ID = "";
                    detail.F1_DISPLAY = item.PackageUnit;
                    detail.ITEM_STANDARD = item.DetectionCodeActual;
                    detail.ITEM_SPEC1 = "";
                    detail.ITEM_SPEC2 = spec2;
                    detail.CHECK_PART = item.TitlePart;
                    detail.CHECK_PART_DISPLAY = item.TitlePart;
                    detail.RESULT = 4;
                    if (item.Judgement == "合格") detail.RESULT = 1;
                    else if (item.Judgement == "不合格")
                    {
                        infoJudement = false;
                        detail.RESULT = 0;
                    }
                    else detail.RESULT_TEXT = "此项无需判定";
                    detail.RESULT_REMARK = result;
                    reportDetail.ITEMS.Add(detail);
                }
                rpt_detail.Add(reportDetail);
                #endregion
            }

            //针对多个报告存在不合格的项目 则回传订单整体判定为不合格
            if (!infoJudement)
            {
                result_info.RESULT = 2;
                result_info.RESULT_TEXT = "不合格";
            }
            if (_client == null) _client = new JsonServiceClient(Configs._defaultUrl);

            var info = JsonSerializer.SerializeToString(result_info);
            var rpt = JsonSerializer.SerializeToString(rpt_detail);

            var body = new Dictionary<string, string>();
            body.Add("result_info", info);
            body.Add("rpt_detail", rpt);

            Console.WriteLine($"result_info:{info}\n");
            Console.WriteLine($"rpt_detail:{rpt}\n");
            var resp = _client.Post<SemirOrderResponse>($"{Configs.RouteGenerateQcResult}?token={Configs._token}", body);
            Console.WriteLine($"Code:{resp.code}\n");
            Console.WriteLine($"Message:{resp.message}\n");
            Console.WriteLine($"resp_code:{resp.data.resp_code}\n");
            Console.WriteLine($"data.message:{resp.data.message}\n");
            Console.WriteLine($"data.errors:{resp.data.errors}\n");
            Console.WriteLine($"recordsTotal:{resp.data.recordsTotal}\n");
            Console.WriteLine($"results:{resp.data.results}\n");
            if (resp.code == 200 || resp.code == 404)
            {
                third.IsUploaded = true;
                third.AddOnlyProperties(o => o.IsUploaded);
                conn.UpdateOnly(third);

                var log = new EntityLog<ThirdReportOrder>();
                log.OwnerKvid = third.Kvid;
                log.BizId = third.SerialNumber;
                log.BizKvid = third.Kvid;
                log.BizType = third.ThirdType;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = $"Code:{resp.code}  Message:{resp.message}";
                conn.Insert(log);
                return true;
            }
            else return false;
        }

        public object OnPullThirdReport()
        {
            #region 设置缓存1分钟内不能连续请求
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey("SemirOrderAccept:LockedOneMinute");
            var cacheValue = cache.Get<string>(cacheKey);
            if (!cacheValue.IsNullOrEmpty())
            {
                throw new Exception(cacheValue);
            }
            #endregion

            var rtns = new RestfulQueryResponse<ThirdReportOrder>();

            if (_client == null) _client = new JsonServiceClient(Configs._defaultUrl);

            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(SemirOrder).FullName);
            //DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime));

            var response = _client.PullSemirOrder(Configs._token, maxTranslateTime);
            cache.Set(cacheKey, "一分钟内不可多次查询!", TimeSpan.FromMinutes(1));

            (response.code != 200).ThrowIfTrue(response.message);
            rtns.Results = new List<ThirdReportOrder>();
            if (response.data == null) return rtns;
            if (response.data.results.IsNullOrEmpty()) return rtns;

            var results = response.data.results.AcceptThirdContractOrder(Guid.Parse(Configs._ownerKvid), Configs._ownerName, conn);
            rtns.Results.AddRange(results);
            rtns.Total = rtns.Results.Count;

            return rtns;
        }
    }
}
